import 'package:ez_localization/ez_localization.dart';
import 'package:flutter/material.dart';
import 'package:flutter_easy_utils/flutter_easy_utils.dart';
import 'package:flutter_easy_utils/src/providers/LocaleProvider.dart';
import 'package:flutter_easy_utils/src/providers/ThemeProvider.dart';
import 'package:flutter_easy_utils/src/providers/UtilsProvider.dart';
import 'package:flutter_easyloading/flutter_easyloading.dart';
import 'package:nested/nested.dart';
import 'package:provider/provider.dart';

/// easy app include some base providers like :
/// utils : for internet check
/// theme : for dark and light
/// locale : for languages
/// ez_localization
/// check updates
/// confirm user age +18
class EasyApp extends StatefulWidget {
  /// builder for home page
  final Widget Function(BuildContext context) builder;

  /// primary color for app
  final Color primary;

  /// accent color for the app
  final Color accent;

  /// providers if your app contain providers
  final List<SingleChildWidget> providers;

  /// check version for updates
  final bool checkVersion;

  /// check age +18
  final bool checkAge;

  @override
  _EasyAppState createState() => _EasyAppState();

  /// builder for the home page widget
  /// primary color for all app
  /// accent color for the app
  /// providers list of the app
  EasyApp(
      {@required this.builder,
      this.primary: Colors.blue,
      this.accent: Colors.deepOrangeAccent,
      this.providers: const [],
      this.checkVersion: true,
      this.checkAge: false});
}

class _EasyAppState extends State<EasyApp> {
  @override
  Widget build(BuildContext context) {
    /// multi provider
    return MultiProvider(
      providers: [
        ChangeNotifierProvider(
          create: (context) => ThemeProvider(),
        ),
        ChangeNotifierProvider(
          create: (context) => LocaleProvider(),
        ),
        ChangeNotifierProvider(
          create: (context) => UtilsProvider(),
        ),
      ]..addAll(widget.providers),
      child: EzLocalizationBuilder(
          delegate: EzLocalizationDelegate(
            supportedLocales: [
              Locale('en'),
              Locale('ar'),
            ],
          ),
          builder: (context, localizationDelegate) {
            return Consumer2<ThemeProvider, LocaleProvider>(
              builder: (context, theme, locale, child) {
                if (theme.isDarkLoaded) {
                  if (locale.isLocaleLoaded) {
                    return MaterialApp(
                      darkTheme: ThemeData(
                        //fontFamily: 'cairo',
                        brightness: Brightness.dark,
                        accentColor: widget.primary,
                        textTheme: TextTheme(
                          headline1: TextStyle(color: Colors.white),
                          headline6: TextStyle(color: Colors.white),
                          bodyText2: TextStyle(color: Colors.white),
                        ),
                      ),
                      themeMode: theme.isDark == null
                          ? ThemeMode.system
                          : (theme.isDark ? ThemeMode.dark : ThemeMode.light),
                      theme: ThemeData(
                        //fontFamily: 'cairo',
                        brightness: Brightness.light,
                        primarySwatch: widget.primary,
                        accentColor: widget.accent,
                      ),

                      /// easy loading for quick toasts
                      builder: EasyLoading.init(),
                      locale: locale.locale,
                      localizationsDelegates:
                          localizationDelegate.localizationDelegates,
                      supportedLocales: localizationDelegate.supportedLocales,
                      localeResolutionCallback:
                          localizationDelegate.localeResolutionCallback,
                      home: Home(
                        builder: widget.builder,
                        checkAge: widget.checkAge,
                        checkVersion: widget.checkVersion,
                      ),
                    );
                  } else {
                    locale.getLocale(context);
                    return IsLoadingWidget();
                  }
                } else {
                  theme.getIsDark();
                  return IsLoadingWidget();
                }
              },
            );
          }),
    );
  }
}

/// get home page for Material App
class Home extends StatefulWidget {
  /// check version for updates
  final bool checkVersion;

  /// check age +18
  final bool checkAge;

  /// builder for home page
  final Widget Function(BuildContext context) builder;

  const Home(
      {Key key,
      this.checkVersion: true,
      this.checkAge: false,
      @required this.builder})
      : super(key: key);

  @override
  _HomeState createState() => _HomeState();
}

class _HomeState extends State<Home> {
  @override
  void initState() {
    /// init the size config for the dimensions
    SizeConfig.init(context);

    /// init check version
    if (widget.checkVersion) {
      checkVersion(context);
    }

    /// init check age
    if (widget.checkAge) {
      confirmAge(context);
    }

    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    /// return your builder app
    return widget.builder(context);
  }
}
