import 'dart:async';
import 'dart:convert';

import 'package:dio/dio.dart';
import 'package:ez_localization/ez_localization.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:rflutter_alert/rflutter_alert.dart';
import 'package:shared_preferences/shared_preferences.dart';

class APIMixin extends Object {
  getWithToken(url) async {
    var token = await getToken();

    if (token == null) {
      return Future.error("no_token");
    }

    Future<Response> response = Dio().get(
      url,
      options: Options(
          contentType: Headers.jsonContentType,
          validateStatus: (status) => true,
          headers: {
            Headers.acceptHeader: Headers.jsonContentType,
          }),
      queryParameters: {"api_token": token},
    );

    return response;
  }

  postWithToken(url, data) async {
    var token = await getToken();

    if (token == null) {
      return Future.error("no_token");
    }

    Future<Response> response = Dio().post(
      url,
      options: Options(
        contentType: Headers.jsonContentType,
        headers: {
          Headers.acceptHeader: Headers.jsonContentType,
        },
        validateStatus: (status) => true,
      ),
      queryParameters: {"api_token": token},
      data: data,
    );

    return response;
  }

  post(url, data) async {
    var response = Dio().post(
      url,
      options: Options(
          contentType: Headers.jsonContentType,
          validateStatus: (status) => true,
          headers: {
            Headers.acceptHeader: Headers.jsonContentType,
          }),
      data: data,
    );

    return response;
  }

  get(url) async {
    Future<Response> response = Dio().get(
      url,
      options: Options(
          contentType: Headers.jsonContentType,
          validateStatus: (status) => true,
          headers: {
            Headers.acceptHeader: Headers.jsonContentType,
          }),
    );

    return response;
  }

  getToken() async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    return prefs.getString('token');
  }

  setToken(token) async {
    SharedPreferences prefs = await SharedPreferences.getInstance();
    return prefs.setString('token', token);
  }

  validateError(context, Map validateErrors) {
    String errors = validateErrors["data"]
        .keys
        .map((e) {
          if (e == null) {
            return null;
          }

          var messages = validateErrors["data"][e].join("\n");

          return "$messages";
        })
        .toList()
        .join("\n");

    return errors;
  }

  falseSuccessError(context, responseJson) {
    var message = responseJson.containsKey("data")
        ? responseJson["data"]
        : (responseJson.containsKey("message")
            ? responseJson["message"]
            : "UNKNOWN_ERROR");
    return EzLocalization.of(context)
        .get('error_server_500', {'message': message});
  }

  noSuccessError(context, responseJson) {
    var message = responseJson.containsKey("data")
        ? responseJson["data"]
        : (responseJson.containsKey("message")
            ? responseJson["message"]
            : "UNKNOWN_ERROR");
    return EzLocalization.of(context)
        .get('error_no_success', {'message': message});
  }

  statusCodeError(context, Response response) {
    Map data = response.data;
    switch (response.statusCode) {
      case 404:
        // not found
        return EzLocalization.of(context).get('error_server_404');
      case 500:
        // server error
        var message = data.containsKey("data")
            ? data["data"]
            : (data.containsKey("message") ? data["message"] : "UNKNOWN_ERROR");
        return EzLocalization.of(context)
            .get('error_server_500', {'message': message});
      case 401:
        // unauth error
        var message = data.containsKey("data")
            ? data["data"]
            : (data.containsKey("message") ? data["message"] : "UNKNOWN_ERROR");
        return EzLocalization.of(context)
            .get('error_server_unauth', {'message': message});
      default:
        print(response);
        var message = data.containsKey("data")
            ? data["data"]
            : (data.containsKey("message") ? data["message"] : "UNKNOWN_ERROR");
        return EzLocalization.of(context)
            .get('error_server_unknown', {'message': message});
    }
  }

  exceptionError(context, e) {
    if (e is DioError) {
      switch (e.type) {
        case DioErrorType.connectTimeout:
          return EzLocalization.of(context).get("connect_timeout");
        case DioErrorType.sendTimeout:
          return EzLocalization.of(context).get("send_timeout");
        case DioErrorType.receiveTimeout:
          return EzLocalization.of(context).get("receive_timeout");
        case DioErrorType.response:
          return EzLocalization.of(context).get("status_error");
        case DioErrorType.cancel:
          return EzLocalization.of(context).get("request_canceled");
        case DioErrorType.other:
        default:
          return EzLocalization.of(context).get("unable_to_connect");
      }
    } else if(e is String){
      if(e == "no_token"){
        return EzLocalization.of(context).get("need_auth");
      }
    }else {
      print(e);
      return EzLocalization.of(context).get("unknown_error" , {"error" : e.toString()});
    }
  }

}